﻿using GH_IO.Serialization;
using Grasshopper;
using Grasshopper.GUI;
using Grasshopper.GUI.Script;
using Grasshopper.Kernel;
using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

/*
 *  HIGHLY based on code by Long Nguyens ScriptSync (and he had some help from David Rutten), Thanks both of you.
 */



namespace DefaultPanel
{
    public partial class GhcDefaultPanel : GH_Component
    {
        private Guid targetPanelComponentGuid = Guid.Empty;
        private Guid sourcePanelComponentGuid = Guid.Empty;

        public GhcDefaultPanel()
            : base(
                "DefaultPanel",
                "DefaultPanel",
                "Defaulting panels in your gh scene",
                "Maths",
                "Script")
        {
        }


        protected override Bitmap Icon => null;
        public override Guid ComponentGuid => new Guid("{e40ccee5-232e-44a0-9624-fb0ea34a833a}");
        public override GH_Exposure Exposure => GH_Exposure.tertiary;
        public override void CreateAttributes() { Attributes = new GhcDefaultPanelAttributes(this); }
        protected override void RegisterInputParams(GH_InputParamManager pManager) { }
        protected override void RegisterOutputParams(GH_OutputParamManager pManager) { }


        protected override void SolveInstance(IGH_DataAccess DA)
        {
            //dynamic targetPanelComponent = OnPingDocument().FindComponent(targetPanelComponentGuid);
            //dynamic sourcePanelComponent = OnPingDocument().FindComponent(sourcePanelComponentGuid);

            var targetPanelComponent = OnPingDocument().FindObject(targetPanelComponentGuid, true);
            var sourcePanelComponent = OnPingDocument().FindObject(sourcePanelComponentGuid, true);

            if (targetPanelComponent == null)
            {
                AddRuntimeMessage(GH_RuntimeMessageLevel.Warning, "Please set a target panel component (in the right-click menu)");
                //return;
            }
            if (sourcePanelComponent == null)
            {
                AddRuntimeMessage(GH_RuntimeMessageLevel.Warning, "Please set a source panel component (in the right-click menu)");
                // return;
            }


        }



        public override bool Write(GH_IWriter writer)
        {
            writer.SetGuid("targetPanelComponentGuid", targetPanelComponentGuid);
            writer.SetGuid("sourcePanelComponentGuid", sourcePanelComponentGuid);

            return base.Write(writer);
        }


        public override bool Read(GH_IReader reader)
        {
            targetPanelComponentGuid = reader.GetGuid("targetScriptComponentGuid");
            sourcePanelComponentGuid = reader.GetGuid("sourceScriptComponentGuid");

            return base.Read(reader);
        }


        public override bool AppendMenuItems(ToolStripDropDown menu)
        {
            Menu_AppendEnableItem(menu);
            Menu_AppendSeparator(menu);
            Menu_AppendRuntimeMessages(menu);
            Menu_AppendSeparator(menu);

            Menu_AppendItem(
                menu,
                targetPanelComponentGuid == Guid.Empty ? "Set Target Panel" : "Change Target Panel",
                delegate { Instances.ActiveCanvas.ActiveInteraction = new GhcDefaultPanelCanvasInteration(Instances.ActiveCanvas, new GH_CanvasMouseEvent(), this, true); }, //true=target
                null,
                true,
                false);

            Menu_AppendItem(
                menu,
                sourcePanelComponentGuid == Guid.Empty ? "Set Source Panel" : "Change Source Panel",
                delegate { Instances.ActiveCanvas.ActiveInteraction = new GhcDefaultPanelCanvasInteration(Instances.ActiveCanvas, new GH_CanvasMouseEvent(), this, false); }, //false=source
                null,
                true,
                false);


            Menu_AppendItem(
                menu,
                "Disconnect Target",
                delegate { targetPanelComponentGuid = Guid.Empty; ExpireSolution(true); },
                null,
                OnPingDocument().FindComponent(targetPanelComponentGuid) != null,
                false);

            Menu_AppendItem(
                menu,
                "Disconnect Source",
                delegate { sourcePanelComponentGuid = Guid.Empty; ExpireSolution(true); },
                null,
                OnPingDocument().FindComponent(sourcePanelComponentGuid) != null,
                false);



            Menu_AppendSeparator(menu);

            Menu_AppendItem(
                menu,
                "Default connected panels(TODO)",
                delegate { sourcePanelComponentGuid = Guid.Empty; ExpireSolution(true); },
                null,
               false,
                false);

            Menu_AppendItem(
                menu,
                "Default ALL panels in file(TODO)",
                delegate { sourcePanelComponentGuid = Guid.Empty; ExpireSolution(true); },
                null,
                false,
                false);


            return true;
        }
    }
}