﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;

using Grasshopper;
using Grasshopper.GUI.Canvas;
using Grasshopper.Kernel;
using Grasshopper.Kernel.Special;

/*
 *  HIGHLY based on code by Long Nguyens ScriptSync (and he had some help from David Rutten), Thanks both of you.
 */

namespace DefaultPanel
{
    public partial class GhcDefaultPanel : GH_Component
    {
        protected class GhcDefaultPanelAttributes : GH_Attributes<GhcDefaultPanel>
        {
            public GhcDefaultPanelAttributes(GhcDefaultPanel owner) : base(owner)
            {
            }

            protected override void Layout()
            {
                Size size = GH_FontServer.MeasureString("DefaultPanel", GH_FontServer.Standard) + new Size(20, 10);
                Bounds = new RectangleF(
                    Pivot.X - 0.5f * size.Width,
                    Pivot.Y - 0.5f * size.Height,
                    size.Width,
                    size.Height);
            }


            protected override void Render(GH_Canvas canvas, Graphics graphics, GH_CanvasChannel channel)
            {
                if (channel == GH_CanvasChannel.Wires)
                {
                    var targetObject = Owner.OnPingDocument().FindObject(Owner.targetPanelComponentGuid, true);
                    var sourceObject = Owner.OnPingDocument().FindObject(Owner.sourcePanelComponentGuid, true);


                    if (targetObject != null)
                    {
                        //Point targetCenter = new Point();
                        RectangleF bounds = targetObject.Attributes.Bounds;
                        PointF targetCenter = new PointF(bounds.X + bounds.Width / 2, bounds.Y + bounds.Height / 2);

                        // The pivot method of Long didnt work. Attempting bounds  - still without luck TODO.
                        //targetCenter.X = Convert.ToInt32(bounds.X);
                        //targetCenter.Y = Convert.ToInt32(bounds.Y);

                        graphics.DrawLine(
                            new Pen(Color.Black, 4f) { DashCap = DashCap.Round, DashPattern = new[] { 1f, 0.25f } },
                            Owner.Attributes.Pivot,
                            targetCenter);
                    }


                    if (sourceObject != null)
                    {
                        //PointF sourceCenter = new PointF();

                        RectangleF bounds = sourceObject.Attributes.Bounds;
                        PointF sourceCenter = new PointF(bounds.X + bounds.Width / 2, bounds.Y + bounds.Height / 2);

                        graphics.DrawLine(
                            new Pen(Color.Black, 2f) { DashCap = DashCap.Round, DashPattern = new[] { 1f, 0.25f } },
                            Owner.Attributes.Pivot,
                            sourceCenter);
                    }

                }
                else if (channel == GH_CanvasChannel.Objects)
                {
                    GH_Palette ghPalette = GH_Palette.Hidden;
                    if (Owner.RuntimeMessageLevel == GH_RuntimeMessageLevel.Warning) ghPalette = GH_Palette.Warning;
                    else if (Owner.RuntimeMessageLevel == GH_RuntimeMessageLevel.Error) ghPalette = GH_Palette.Error;

                    GH_Capsule ghTextCapsule = GH_Capsule.CreateTextCapsule(
                        GH_Convert.ToRectangle(Bounds),
                        GH_Convert.ToRectangle(Bounds),
                        ghPalette,
                        "DefaultPanel 0001",
                        GH_FontServer.Standard,
                        GH_Orientation.horizontal_center,
                        (int)Bounds.Height / 2,
                        0);

                    ghTextCapsule.Render(graphics, null, Selected, Owner.Locked, false);
                }
            }
        }
    }
}